<div class="chatlog__message-group">
    {{REFERENCE}}
    <div class="chatlog__author-avatar-container">
        <img class="chatlog__author-avatar" src="{{AVATAR_URL}}" />
    </div>
    <div class="chatlog__messages">
        <span class="chatlog__author-name" title="{{NAME_TAG}}" data-user-id="{{USER_ID}}" style="{{USER_COLOUR}}">{{NAME}}</span>

        {{BOT_TAG}}

        <span class="chatlog__timestamp">{{TIMESTAMP}}</span>