<div class="chatlog__embed-author">
                    <img class="chatlog__embed-author-icon" src="{{AUTHOR_ICON}}" alt="Author Icon">
                    <span class="chatlog__embed-author-name">{{AUTHOR}}</span>
</div>