<div class="chatlog__reference-symbol" xmlns="http://www.w3.org/1999/html">
</div>
<div class="chatlog__reference">
    <img class="chatlog__reference-avatar" src="{{AVATAR_URL}}" alt="Avatar">
    {{BOT_TAG}}
    <span class="chatlog__reference-name" title="{{NAME_TAG}}" style="{{USER_COLOUR}}">{{NAME}}</span>
    <a class="chatlog__reference-link" href="#" onclick="scrollToMessage(event, '{{MESSAGE_ID}}')">
       <span class="chatlog__reference-content">{{CONTENT}}</span>
        {{EDIT}}
        {{ATTACHMENT_ICON}}
    </a>
</div>